@@ -2,9 +2,9 @@
 # -*- coding: latin-1 -*-
 
 """
-RiceRocks (Asteroids) (June 22, 2013)
+RiceRocks (Asteroids) (June 27, 2013)
 
-My solution (slightly retouched) of the mini-project #8 of the course
+My slightly retouched solution of the mini-project #8 of the course
 https://www.coursera.org/course/interactivepython (Coursera 2013).
 
 Run on (maybe very slow on some browsers):
@@ -24,15 +24,20 @@ import math
 import random
 
 try:
-    from user16_v0hIgQGF5JqtOUQ import Loader
+    from user17_wvnbup0PNzbGTlS import Loader
 
     import simplegui
+
+    SIMPLEGUICS2PYGAME = False
 except:
     from SimpleGUICS2Pygame.simplegui_lib import Loader
 
     import SimpleGUICS2Pygame.simpleguics2pygame as simplegui
 
     simplegui.Frame._hide_status = True
+    simplegui.Frame._keep_timers = False
+
+    SIMPLEGUICS2PYGAME = True
 
 
 #
@@ -57,7 +62,7 @@ def angle_to_vector(angle):
     """
     Return the vector corresponding to the angle expressed in radians.
 
-    :param: int or float
+    :param angle: int or float
 
     :return: (-1 <= float <= 1, -1 <= float <= 1)
     """
@@ -101,6 +106,20 @@ def assert_position(position, non_negative=False, non_zero=False):
         assert position[1] != 0, position
 
 
+def vector_to_angle(vector):
+    """
+    Return the angle (in radians) corresponding to the direction of vector.
+
+    :param vector: (int or float, int or float)
+                   or [int or float, int or float]
+
+    :return: -math.pi <= float <= math.pi
+    """
+    assert_position(vector)
+
+    return math.atan2(vector[1], vector[0])
+
+
 #
 # Classes
 ##########
@@ -128,6 +147,8 @@ class RiceRocks:
         self.missiles = []
         self.rocks = []
 
+        self.music_active = True
+        self.sounds_active = True
         self.timer = simplegui.create_timer(1000, self.rock_spawner)
 
     def bomb_explode(self):
@@ -137,8 +158,9 @@ class RiceRocks:
         """
         if self.nb_bombs:
             self.nb_bombs -= 1
-            self.medias.get_sound('bomb_explode').rewind()
-            self.medias.get_sound('bomb_explode').play()
+            if self.sounds_active:
+                self.medias.get_sound('bomb_explode').rewind()
+                self.medias.get_sound('bomb_explode').play()
             for rock in self.rocks:
                 self.explosions.append(Sprite(rock.position, rock.velocity,
                                               0, rock.angle_velocity,
@@ -154,11 +176,12 @@ class RiceRocks:
         self.time += 1
 
         # Draw static background
-        canvas.draw_image(self.medias.get_image('nebula'),
-                          self.img_infos['nebula'].get_center(),
-                          self.img_infos['nebula'].get_size(),
-                          (SCREEN_WIDTH/2.0, SCREEN_HEIGHT/2.0),
-                          (SCREEN_WIDTH, SCREEN_HEIGHT))
+        if not SIMPLEGUICS2PYGAME:
+            canvas.draw_image(self.medias.get_image('nebula'),
+                              self.img_infos['nebula'].get_center(),
+                              self.img_infos['nebula'].get_size(),
+                              (SCREEN_WIDTH/2.0, SCREEN_HEIGHT/2.0),
+                              (SCREEN_WIDTH, SCREEN_HEIGHT))
 
         # Draw animated background
         center = self.img_infos['debris'].get_center()
@@ -215,12 +238,43 @@ class RiceRocks:
                         del self.missiles[i]
                         del self.rocks[j]
 
+                        if not rock.little:
+                            # Divide in two little asteroids
+                            angle = vector_to_angle(missile.velocity)
+                            mvel = math.sqrt(
+                                (rock.velocity[0]*rock.velocity[0]
+                                 + rock.velocity[1]*rock.velocity[1])/2.0)
+
+                            vel = list(angle_to_vector(angle - math.pi/4))
+                            vel[0] *= mvel
+                            vel[1] *= mvel
+                            little1 = Asteroid(rock.position, vel,
+                                               rock.angle_velocity*2,
+                                               rock.num, True)
+
+                            vel = list(angle_to_vector(angle + math.pi/4))
+                            vel[0] *= mvel
+                            vel[1] *= mvel
+                            little2 = Asteroid(rock.position, vel,
+                                               rock.angle_velocity*2,
+                                               rock.num, True)
+
+                            while True:
+                                little1.position[0] += little1.velocity[0]
+                                little1.position[1] += little1.velocity[1]
+                                little2.position[0] += little2.velocity[0]
+                                little2.position[1] += little2.velocity[1]
+                                if not little1.collide(little2):
+                                    break
+                            self.rocks.extend((little1, little2))
+
                         self.score += 1
                         if self.score % 10 == 0:  # add a new bomb
                             self.nb_bombs += 1
-                            self.medias.get_sound('bomb_extra').rewind()
-                            self.medias.get_sound('bomb_extra').play()
-                            if self.score % 100 == 0:  # add a new live
+                            if self.sounds_active:
+                                self.medias.get_sound('bomb_extra').rewind()
+                                self.medias.get_sound('bomb_extra').play()
+                            if self.score % 50 == 0:  # add a new live
                                 self.lives += 1
 
                         self.explosions.append(Sprite(rock.position,
@@ -256,13 +310,49 @@ class RiceRocks:
                                                   self.my_ship.angle_velocity,
                                                   'ship_explosion'))
 
-                    self.medias.get_sound('death').rewind()
-                    self.medias.get_sound('death').play()
+                    if self.sounds_active:
+                        self.medias.get_sound('death').rewind()
+                        self.medias.get_sound('death').play()
 
                     break
                 else:
-                    self.medias.get_sound('collide').rewind()
-                    self.medias.get_sound('collide').play()
+                    if self.sounds_active:
+                        self.medias.get_sound('collide').rewind()
+                        self.medias.get_sound('collide').play()
+            else:
+                for j in range(0, i):
+                    other = self.rocks[j]
+                    if rock.collide(other):
+                        rock.position[0] = (rock.position[0]
+                                            - rock.velocity[0]) % SCREEN_WIDTH
+                        rock.position[1] = (rock.position[1]
+                                            - rock.velocity[1]) % SCREEN_HEIGHT
+
+                        other.position[0] = ((other.position[0]
+                                              - other.velocity[0])
+                                             % SCREEN_WIDTH)
+                        other.position[1] = ((other.position[1]
+                                              - other.velocity[1])
+                                             % SCREEN_HEIGHT)
+
+                        # Elastic collision (with radius*3 as mass)
+                        sum = (rock.radius + other.radius)*3
+                        diff = (rock.radius - other.radius)*3
+
+                        double = 2*3*other.radius
+                        new_x = (float(diff*rock.velocity[0]
+                                       + double*other.velocity[0])/sum)
+                        new_y = (float(diff*rock.velocity[1]
+                                       + double*other.velocity[1])/sum)
+
+                        double = 2*3*rock.radius
+                        other.velocity[0] = float(double*rock.velocity[0]
+                                                  - diff*other.velocity[0])/sum
+                        other.velocity[1] = float(double*rock.velocity[1]
+                                                  - diff*other.velocity[1])/sum
+
+                        rock.velocity[0] = new_x
+                        rock.velocity[1] = new_y
 
         # Display number of lives
         if self.started:
@@ -337,6 +427,15 @@ class RiceRocks:
                                                                   True),
                           'bomb': ImageInfo((10, 10), (20, 20)),
                           'debris': ImageInfo((320, 240), (640, 480)),
+                          'little-asteroid-1': ImageInfo((45, 45), (90, 90),
+                                                         27, None, False,
+                                                         (60, 60)),
+                          'little-asteroid-2': ImageInfo((45, 45), (90, 90),
+                                                         27, None, False,
+                                                         (60, 60)),
+                          'little-asteroid-3': ImageInfo((45, 45), (90, 90),
+                                                         26, None, False,
+                                                         (60, 60)),
                           'live_explosion': ImageInfo((64, 64), (128, 128),
                                                       17, 24, True),
                           'missile': ImageInfo((5, 5), (10, 10), 3, 50),
@@ -346,7 +445,42 @@ class RiceRocks:
                                                       17, 24, True),
                           'splash': ImageInfo((200, 150), (400, 300))}
 
-        self.medias = Loader()
+        def init():
+            """
+            Init the game after medias loaded.
+            """
+            if SIMPLEGUICS2PYGAME:
+                frame._set_canvas_background_image(
+                    self.medias.get_image('nebula'))
+
+            self.medias._images['live_explosion'] = \
+                self.medias._images['ship_explosion']
+
+            for i in range(1, 4):
+                self.medias._images['little-asteroid-' + str(i)] = \
+                    self.medias._images['asteroid-' + str(i)]
+
+            self.medias._sounds['asteroid_collide_explosion'] = \
+                self.medias._sounds['asteroid_explosion']
+
+            self.medias.get_sound('missile').set_volume(.5)
+
+            self.my_ship = Ship((SCREEN_WIDTH/2.0, SCREEN_HEIGHT/2.0), (0, 0),
+                                -math.pi/2, 'ship')
+
+            frame.set_draw_handler(self.draw_and_update)
+
+            frame.set_keydown_handler(keydown)
+            frame.set_keyup_handler(keyup)
+
+            frame.set_mouseclick_handler(click)
+
+            if ricerocks.music_active:
+                self.medias.get_sound('intro').play()
+
+            self.loaded = True
+
+        self.medias = Loader(frame, SCREEN_WIDTH, init)
 
         # Images by Kim Lathrop
         self.medias.add_image('http://commondatastorage.googleapis.com/codeskulptor-assets/lathrop/asteroid_blend.png',
@@ -396,33 +530,7 @@ class RiceRocks:
 
         self.medias.load()
 
-        def init():
-            """
-            Init the game after medias loaded.
-            """
-            self.medias.get_sound('missile').set_volume(.5)
-
-            self.my_ship = Ship((SCREEN_WIDTH/2.0, SCREEN_HEIGHT/2.0), (0, 0),
-                                -math.pi/2, 'ship')
-
-            frame.set_draw_handler(self.draw_and_update)
-
-            frame.set_keydown_handler(keydown)
-            frame.set_keyup_handler(keyup)
-
-            frame.set_mouseclick_handler(click)
-
-            self.medias.get_sound('intro').play()
-
-            self.medias._images['live_explosion'] = \
-                self.medias._images['ship_explosion']
-
-            self.medias._sounds['asteroid_collide_explosion'] = \
-                self.medias._sounds['asteroid_explosion']
-
-            self.loaded = True
-
-        self.medias.wait_loaded(frame, SCREEN_WIDTH, init)
+        self.medias.wait_loaded()
 
     def rock_spawner(self):
         """
@@ -436,31 +544,43 @@ class RiceRocks:
                 """
                 :return: int or float
                 """
-                return min(10,
+                return min(3,
                            (random.random()*0.3*(
                             self.score/2 + 1)))*random.choice((-1, 1))
 
-            while too_close:
+            for i in range(10):
                 rock_pos = (random.randrange(0, SCREEN_WIDTH),
                             random.randrange(0, SCREEN_HEIGHT))
                 rock_vel = (random_vel(),
                             random_vel())
                 rock_ang_vel = random.random()*0.2 - 0.1
 
-                rock = Sprite(rock_pos, rock_vel,
-                              0, rock_ang_vel,
-                              'asteroid-' + str(random.randint(1, 3)))
-                too_close = (self.my_ship.distance(rock)
-                             < (self.my_ship.radius + rock.radius)*1.5)
+                rock = Asteroid(rock_pos, rock_vel,
+                                rock_ang_vel,
+                                random.randint(1, 3))
 
-            self.rocks.append(rock)
+                too_close = False
+                for r in self.rocks:
+                    if r.collide(rock):
+                        too_close = True
+
+                        break
+                too_close = (too_close
+                             or (self.my_ship.distance(rock)
+                                 < (self.my_ship.radius + rock.radius)*3))
+                if not too_close:
+                    break
+
+            if not too_close:
+                self.rocks.append(rock)
 
     def start(self):
         """
         Start the game.
         """
-        self.medias.get_sound('intro').rewind()
-        self.medias.get_sound('soundtrack').play()
+        if ricerocks.music_active:
+            self.medias.get_sound('intro').rewind()
+            self.medias.get_sound('soundtrack').play()
 
         self.keydown_left = False
         self.keydown_right = False
@@ -493,10 +613,11 @@ class RiceRocks:
 
         self.started = False
 
-        self.medias.get_sound('soundtrack').rewind()
         self.my_ship.stop()
 
-        self.medias.get_sound('intro').play()
+        if ricerocks.music_active:
+            self.medias.get_sound('soundtrack').rewind()
+            self.medias.get_sound('intro').play()
 
 
 class ImageInfo:
@@ -504,7 +625,8 @@ class ImageInfo:
     Informations to use with Sprite.
     """
     def __init__(self, center, size,
-                 radius=None, lifespan=None, animated=False):
+                 radius=None, lifespan=None, animated=False,
+                 draw_size=None):
         """
         Set informations.
 
@@ -518,6 +640,9 @@ class ImageInfo:
         :param radius: None or ((int or float) > 0)
         :param lifespan: None or ((int or float) > 0)
         :param animated: bool
+        :param draw_size: None
+                          or ((int or float) > 0, (int or float) > 0)
+                          or [(int or float) > 0, (int or float) > 0]
         """
         assert_position(center)
         assert_position(size, True, True)
@@ -529,6 +654,11 @@ class ImageInfo:
                     and (lifespan > 0))), lifespan
         assert isinstance(animated, bool), type(animated)
 
+        if draw_size is None:
+            draw_size = size
+
+        assert_position(draw_size, True, True)
+
         self._center = list(center)
         self._size = list(size)
         self._radius = (max(size) if radius is None
@@ -536,6 +666,7 @@ class ImageInfo:
         self._lifespan = (lifespan if lifespan
                           else float('inf'))
         self._animated = animated
+        self._draw_size = list(draw_size)
 
     def get_animated(self):
         """
@@ -555,6 +686,14 @@ class ImageInfo:
         """
         return list(self._center)
 
+    def get_draw_size(self):
+        """
+        Return draw size of image.
+
+        :return: [(int or float) > 0, (int or float) > 0]
+        """
+        return list(self._draw_size)
+
     def get_lifespan(self):
         """
         Return lifespan of image.
@@ -584,8 +723,8 @@ class Sprite:
     """
     Sprite class
     """
-    def __init__(self, position, velocity, angle,
-                 angle_velocity,
+    def __init__(self, position, velocity,
+                 angle, angle_velocity,
                  media_name):
         """
         Set sprite.
@@ -595,6 +734,7 @@ class Sprite:
         :param velocity: (int or float, int or float)
                          or [int or float, int or float]
         :param angle: int or float
+        :param angle_velocity: int or float
         :param media_name: str
         """
         assert_position(position)
@@ -604,7 +744,8 @@ class Sprite:
                 or isinstance(angle_velocity, float)), type(angle_velocity)
         assert isinstance(media_name, str), type(media_name)
 
-        if media_name in ricerocks.medias._sounds:
+        if (ricerocks.sounds_active
+                and (media_name in ricerocks.medias._sounds)):
             sound = ricerocks.medias.get_sound(media_name)
             sound.rewind()
             sound.play()
@@ -618,6 +759,7 @@ class Sprite:
         img_info = ricerocks.img_infos[media_name]
         self.animated = img_info.get_animated()
         self.image_center = img_info.get_center()
+        self.image_draw_size = img_info.get_draw_size()
         self.image_size = img_info.get_size()
         self.lifespan = img_info.get_lifespan()
         self.radius = img_info.get_radius()
@@ -661,7 +803,7 @@ class Sprite:
         if self.image.get_width() > 0:
             canvas.draw_image(self.image,
                               self.image_center, self.image_size,
-                              self.position, self.image_size,
+                              self.position, self.image_draw_size,
                               self.angle)
         else:
             # Useful to debug
@@ -688,11 +830,46 @@ class Sprite:
                 self.image_center[0] += self.image_size[0]
 
 
+class Asteroid(Sprite):
+    """
+    Asteroid class
+    """
+    def __init__(self, position, velocity,
+                 angle_velocity,
+                 num, little=False):
+        """
+        Set an asteroid sprite.
+
+        :param position: (int or float, int or float)
+                         or [int or float, int or float]
+        :param velocity: (int or float, int or float)
+                         or [int or float, int or float]
+        :param angle_velocity: int or float
+        :param num: 1, 2 or 3
+        :param litle: bool
+        """
+        assert_position(position)
+        assert_position(velocity)
+        assert (isinstance(angle_velocity, int)
+                or isinstance(angle_velocity, float)), type(angle_velocity)
+        assert num in (1, 2, 3)
+        assert isinstance(little, bool), type(little)
+
+        Sprite.__init__(self, position, velocity,
+                        0, angle_velocity,
+                        ('little-asteroid-' if little
+                         else 'asteroid-') + str(num))
+
+        self.num = num
+        self.little = little
+
+
 class Ship(Sprite):
     """
     Ship class
     """
-    def __init__(self, position, velocity, angle,
+    def __init__(self, position, velocity,
+                 angle,
                  media_name):
         """
         Set ship sprite.
@@ -752,11 +929,13 @@ class Ship(Sprite):
         self.thrust = not self.thrust
 
         if self.thrust:
-            ricerocks.medias.get_sound('ship_thrust').play()
+            if ricerocks.sounds_active:
+                ricerocks.medias.get_sound('ship_thrust').play()
             # Sprite image with actif thrust
             self.image_center[0] += self.image_size[0]
         else:
-            ricerocks.medias.get_sound('ship_thrust').rewind()
+            if ricerocks.sounds_active:
+                ricerocks.medias.get_sound('ship_thrust').rewind()
             # Sprite image with inactif thrust
             self.image_center[0] -= self.image_size[0]
 
@@ -857,6 +1036,8 @@ def keyup(key):
                                    else None)
         elif key == simplegui.KEY_MAP['up']:
             ricerocks.my_ship.thrust_on_off()
+        elif key == 27:  # Escape
+            quit()
 
 
 def quit():
@@ -876,6 +1057,33 @@ def stop():
         ricerocks.stop()
 
 
+def switch_music():
+    """
+    Switch music on/off.
+    """
+    ricerocks.music_active = not ricerocks.music_active
+
+    if ricerocks.music_active:
+        button_music.set_text('Music off')
+        if ricerocks.started:
+            ricerocks.medias.get_sound('soundtrack').play()
+        else:
+            ricerocks.medias.get_sound('intro').play()
+    else:
+        button_music.set_text('Music on')
+        ricerocks.medias.get_sound('intro').rewind()
+        ricerocks.medias.get_sound('soundtrack').rewind()
+
+
+def switch_sounds():
+    """
+    Switch sounds on/off.
+    """
+    ricerocks.sounds_active = not ricerocks.sounds_active
+    button_sounds.set_text('Sounds off' if ricerocks.sounds_active
+                           else 'Sounds on')
+
+
 #
 # Main
 #######
@@ -888,6 +1096,9 @@ if __name__ == '__main__':
 
     frame.add_button('Stop this game', stop)
     frame.add_label('')
+    button_music = frame.add_button('Music off', switch_music)
+    button_sounds = frame.add_button('Sounds off', switch_sounds)
+    frame.add_label('')
     frame.add_button('Quit', quit)
     frame.add_label('')
     frame.add_label('Turn: Left and Right')
@@ -895,10 +1106,11 @@ if __name__ == '__main__':
     frame.add_label('Flip: Down')
     frame.add_label('Fire: Space')
     frame.add_label('Bomb: Z (or W)')
+    frame.add_label('Esc: Quit')
 
     frame.add_label('')
     frame.add_label('One bomb for every 10 asteroids destroyed.')
     frame.add_label('')
-    frame.add_label('One live for every 100 asteroids destroyed.')
+    frame.add_label('One live for every 50 asteroids destroyed.')
 
     frame.start()
